#region Using declarations
using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	public class spsfDevelopingPOC : Indicator
	{
		public class RowData
		{
			public long TotalVolume;
			public double Price;
			
			public RowData(long totalVolume, double price)
			{
				TotalVolume = totalVolume;
				Price = price;				
			}
		}		
		
		public class Profile
		{
			public Dictionary<double, RowData> Data;
			public long PocValue = 0;
			public double PocPrice = 0;
			public long ProfileTotalVolume = 0;
			public double TickSize;

			public Profile(double tickSize)
			{
				TickSize = tickSize;
				
				Data = new Dictionary<double, RowData>();
				ProfileTotalVolume = 0;
			}
			
			public void AddVolume(double price, long volume)
			{
				ProfileTotalVolume += volume;
				
				RowData val;
				
				if (Data.TryGetValue(price, out val))
			    {
					var total = val.TotalVolume + volume;
					val.TotalVolume = total;	
					
					if(total > PocValue)
					{
						PocValue = total;
						PocPrice = price;
					}
			    }
			    else
			    {
			        Data.Add(price, new RowData(volume, price));
					
					if(volume > PocValue)
					{
						PocValue = volume;
						PocPrice = price;
					}
			    }		
			}
		}		
		
		private SessionIterator sessionIterator;
		private Profile myProfile;
		private bool firstDay = true;
		
		private Dictionary<DateTime, Double> _cache = new Dictionary<DateTime, Double>();
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"The point of control or price with most volume for that bar.";
				Name										= "Developing POC";
				Calculate									= Calculate.OnEachTick;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= false;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= false;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= false;
				
				CachePath = "DevelopingPOC";
				
				AddPlot(new Stroke(Brushes.Yellow, DashStyleHelper.Solid, 3), PlotStyle.Hash, "POC");
			}
			else if (State == State.Configure)
			{
				sessionIterator = new SessionIterator(Bars);
			}
			else if (State == State.DataLoaded)
			{
				 myProfile = new Profile(TickSize);	
			}
		}
		
		protected override void OnMarketData(MarketDataEventArgs e)
		{
			if(CurrentBar < 0)
				return;
			
			
			if (e.MarketDataType == MarketDataType.Last)
			{
				myProfile.AddVolume(e.Price, e.Volume);	
			}
		}

		protected override void OnBarUpdate()
		{		
			if (CurrentBar == 0) return;
			
			DateTime previous = sessionIterator.GetTradingDay(Time[1]);
			DateTime current = sessionIterator.GetTradingDay(Time[0]);

			if (CurrentBar == 1 && IsFirstTickOfBar)
			{
				Import(current);
				return;
			}

			
			if (!current.DayOfWeek.Equals(previous.DayOfWeek))
			{
				NewDay(previous, current);
			}
			else if (IsFirstTickOfBar)
			{
				Double val;
				
				if (_cache.TryGetValue(Time[0], out val))
				{
					POC[0] = val;
				}
				else
				{
					POC[0] = myProfile.PocPrice;
					_cache[Time[0]] = POC[0];
				}
			}
			else
			{
				POC[0] = myProfile.PocPrice;
				_cache[Time[0]] = POC[0];
			}
			
		}
		
		protected void NewDay(DateTime previous, DateTime current)
		{
			Export(previous);
			Import(current);
			
			
			myProfile = new Profile(TickSize);			
		}
		
		protected void Export(DateTime dateTime)
		{
			try
			{
				string date = dateTime.ToShortDateString();
				string filename = NinjaTrader.Core.Globals.UserDataDir + "cache\\" + CachePath + "\\" + date + ".csv";

				if(!File.Exists(filename))
				{
					Print("Exporting to: " + filename);
					using(StreamWriter sw = new StreamWriter(filename))
					{
						foreach(KeyValuePair<DateTime, double> entry in _cache)
						{
							string poc = POC[0].ToString();
							string data = string.Format("{0}, {1}", entry.Key, entry.Value);
							sw.WriteLine(data);
						}
					}
				}
			}
			catch(Exception e)
			{
				Print("Error exporting data to file " + e.Message);
			}
		}
		
		protected void Import(DateTime current)
		{
			_cache = new Dictionary<DateTime, Double>();
			string date = current.ToShortDateString();
			string folder = NinjaTrader.Core.Globals.UserDataDir + "cache\\" + CachePath + "\\";
			System.IO.Directory.CreateDirectory(folder);
			string filename = folder + date + ".csv";

			if(File.Exists(filename))
			{
				Print("Importing: " + filename);
				try{
					using(StreamReader sr = new StreamReader(filename))
					{
						string line;
					
						while ((line = sr.ReadLine()) != null) 
		                {
							int comma = line.IndexOf(',');
							DateTime entrydate = DateTime.Parse(line.Substring(0, comma));					
						
							string doubles = line.Substring(comma+1);
							double data = Convert.ToDouble(doubles);						
						
							_cache.Add(entrydate, data);
						}
					}
				}
				catch(FileNotFoundException e)
				{
					Print("Cache file not found: " + folder + CachePath);
				}
			}
		}
		
		protected bool HasData(DateTime date)
		{
			return _cache.ContainsKey(date);
		}		
		
		public String CachePath
		{ get; set; }
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> POC
		{
			get { return Values[0]; }
		}
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfDevelopingPOC[] cachespsfDevelopingPOC;
		public SpeculatorSethFree.spsfDevelopingPOC spsfDevelopingPOC()
		{
			return spsfDevelopingPOC(Input);
		}

		public SpeculatorSethFree.spsfDevelopingPOC spsfDevelopingPOC(ISeries<double> input)
		{
			if (cachespsfDevelopingPOC != null)
				for (int idx = 0; idx < cachespsfDevelopingPOC.Length; idx++)
					if (cachespsfDevelopingPOC[idx] != null &&  cachespsfDevelopingPOC[idx].EqualsInput(input))
						return cachespsfDevelopingPOC[idx];
			return CacheIndicator<SpeculatorSethFree.spsfDevelopingPOC>(new SpeculatorSethFree.spsfDevelopingPOC(), input, ref cachespsfDevelopingPOC);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfDevelopingPOC spsfDevelopingPOC()
		{
			return indicator.spsfDevelopingPOC(Input);
		}

		public Indicators.SpeculatorSethFree.spsfDevelopingPOC spsfDevelopingPOC(ISeries<double> input )
		{
			return indicator.spsfDevelopingPOC(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfDevelopingPOC spsfDevelopingPOC()
		{
			return indicator.spsfDevelopingPOC(Input);
		}

		public Indicators.SpeculatorSethFree.spsfDevelopingPOC spsfDevelopingPOC(ISeries<double> input )
		{
			return indicator.spsfDevelopingPOC(input);
		}
	}
}

#endregion
