#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Collections.Generic;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	public class OrderStruct
	{
		public OrderStruct(double price, DateTime time)
		{
			Time = time;
			Price = price;
		}
		
		public DateTime Time
		{get; set;}
		
		public double Price
		{get; set;}
	}
	
	
	public class spsfHaultBands : Indicator
	{
		private List<OrderStruct> _window = new List<OrderStruct>();
	
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "HaultBands";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				Period = 10;
				Percent = 5;
				
				AddPlot(Brushes.Yellow, "Mean");
				AddPlot(Brushes.Yellow, "Up");
				AddPlot(Brushes.Yellow, "Down");
			}
			else if (State == State.Configure)
			{
			}
		}

		protected override void OnBarUpdate()
		{
			//Add your custom indicator logic here.
			_window.RemoveAll(x=> x.Time <= Time[0].AddMinutes(-Period));
			
			double sum = 0;
			
			foreach(OrderStruct o in _window)
			{
				sum = sum + o.Price;
			}
			
			Mean[0] = sum / _window.Count;
			
			double percent = Percent * .01;
			Up[0] = Mean[0] * (1 + percent);
			Down[0] = Mean[0] * (1 - percent);
		}
		
		protected override void OnMarketData(MarketDataEventArgs e)
		{
			if (e.MarketDataType == MarketDataType.Last)
			{
				_window.Add(new OrderStruct(e.Price, e.Time));
			}
		}
		
		[NinjaScriptProperty]
		public int Period
		{get; set;}
		
		[NinjaScriptProperty]
		public int Percent
		{get; set;}
		
		[XmlIgnore]
		[Browsable(false)]	
		public Series<double> Mean
		{
			get { return Values[0]; }
		}	
		
		[XmlIgnore]
		[Browsable(false)]	
		public Series<double> Up
		{
			get { return Values[1]; }
		}	
		
		[XmlIgnore]
		[Browsable(false)]	
		public Series<double> Down
		{
			get { return Values[2]; }
		}	
		
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfHaultBands[] cachespsfHaultBands;
		public SpeculatorSethFree.spsfHaultBands spsfHaultBands(int period, int percent)
		{
			return spsfHaultBands(Input, period, percent);
		}

		public SpeculatorSethFree.spsfHaultBands spsfHaultBands(ISeries<double> input, int period, int percent)
		{
			if (cachespsfHaultBands != null)
				for (int idx = 0; idx < cachespsfHaultBands.Length; idx++)
					if (cachespsfHaultBands[idx] != null && cachespsfHaultBands[idx].Period == period && cachespsfHaultBands[idx].Percent == percent && cachespsfHaultBands[idx].EqualsInput(input))
						return cachespsfHaultBands[idx];
			return CacheIndicator<SpeculatorSethFree.spsfHaultBands>(new SpeculatorSethFree.spsfHaultBands(){ Period = period, Percent = percent }, input, ref cachespsfHaultBands);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfHaultBands spsfHaultBands(int period, int percent)
		{
			return indicator.spsfHaultBands(Input, period, percent);
		}

		public Indicators.SpeculatorSethFree.spsfHaultBands spsfHaultBands(ISeries<double> input , int period, int percent)
		{
			return indicator.spsfHaultBands(input, period, percent);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfHaultBands spsfHaultBands(int period, int percent)
		{
			return indicator.spsfHaultBands(Input, period, percent);
		}

		public Indicators.SpeculatorSethFree.spsfHaultBands spsfHaultBands(ISeries<double> input , int period, int percent)
		{
			return indicator.spsfHaultBands(input, period, percent);
		}
	}
}

#endregion
