#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	[Gui.CategoryOrder("Parameters", 1)]
	[Gui.CategoryOrder("Display", 15)]
	[Gui.CategoryOrder("Data Series", 20)]
	[Gui.CategoryOrder("Setup", 30)]
	[Gui.CategoryOrder("Visual", 40)]
	[Gui.CategoryOrder("Plots", 50)]
	[Gui.CategoryOrder("Version", 80)]	
	public class spsfLineAtTime : Indicator
	{
		private string	versionString = "v 1.0  -  Sept 15, 2022";
		private string BellFilename;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "LineAtTime";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				Hour = 7;
				Minute = 30;
				
				LineColor = Brushes.Gray;
				LineStyle = DashStyleHelper.Dot;
				LineWidth = 4;
			}
			else if (State == State.Configure)
			{
			}
			if (State == State.Historical)
			{
			      // Make sure our object plots behind the chart bars
				SetZOrder(-1);
			}
			else if (State == State.DataLoaded)
			{
				SetZOrder(-1);
				Name = "";
				
				BellFilename = NinjaTrader.Core.Globals.UserDataDir + Bell;
			}
		}

		protected override void OnBarUpdate()
		{
			if( Time[0].Hour == Hour && Time[0].Minute == Minute )
			{
				VerticalLine v = Draw.VerticalLine(this, "Session Start" + CurrentBar, Time[0], LineColor, LineStyle, LineWidth);
				v.ZOrder = -1;
				
				if(State == State.Realtime && Bell != "")
				{
					Print(BellFilename);
					PlaySound(BellFilename);
				}
			}			
		}
		
		
		[Range(0, int.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="StartHour", Description="Hour of Slot Start", Order=3, GroupName="Parameters")]
		public int Hour
		{ get; set; }
		
		[Range(0, int.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="StartMinute", Description="Minute of Slot Stop", Order=4, GroupName="Parameters")]
		public int Minute
		{ get; set; }
		
		[XmlIgnore()]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Time Slot Color", Description = "Slot Color", GroupName = "Parameters", Order = 7)]
        public System.Windows.Media.Brush LineColor
        { get; set; }

		
        [Browsable(false)]
        public string LineColorSerialize
        {
            get { return Serialize.BrushToString(LineColor); }
            set { LineColor = Serialize.StringToBrush(value); }
        }
		
		[Display(ResourceType = typeof(Custom.Resource), Name = "Line Style", Description = "Line Style", GroupName = "Parameters", Order = 7)]
		public DashStyleHelper LineStyle
		{ get; set; }
		
		[Display(ResourceType = typeof(Custom.Resource), Name = "Line Width", Description = "Line Width", GroupName = "Parameters", Order = 7)]
		public int LineWidth
		{ get; set; }
		
		[Display(Name = "Bell Location", Description = "Name of sound file in Ninjatrader8 directory", GroupName = "Parameters", Order = 8)]
		public string Bell
		{ get; set; }

		[XmlIgnore]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Release and date", Description = "Release and date", GroupName = "Version", Order = 0)]
		public string VersionString
		{	
            get { return versionString; }
            set { ; }
		}		
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfLineAtTime[] cachespsfLineAtTime;
		public SpeculatorSethFree.spsfLineAtTime spsfLineAtTime(int hour, int minute)
		{
			return spsfLineAtTime(Input, hour, minute);
		}

		public SpeculatorSethFree.spsfLineAtTime spsfLineAtTime(ISeries<double> input, int hour, int minute)
		{
			if (cachespsfLineAtTime != null)
				for (int idx = 0; idx < cachespsfLineAtTime.Length; idx++)
					if (cachespsfLineAtTime[idx] != null && cachespsfLineAtTime[idx].Hour == hour && cachespsfLineAtTime[idx].Minute == minute && cachespsfLineAtTime[idx].EqualsInput(input))
						return cachespsfLineAtTime[idx];
			return CacheIndicator<SpeculatorSethFree.spsfLineAtTime>(new SpeculatorSethFree.spsfLineAtTime(){ Hour = hour, Minute = minute }, input, ref cachespsfLineAtTime);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfLineAtTime spsfLineAtTime(int hour, int minute)
		{
			return indicator.spsfLineAtTime(Input, hour, minute);
		}

		public Indicators.SpeculatorSethFree.spsfLineAtTime spsfLineAtTime(ISeries<double> input , int hour, int minute)
		{
			return indicator.spsfLineAtTime(input, hour, minute);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfLineAtTime spsfLineAtTime(int hour, int minute)
		{
			return indicator.spsfLineAtTime(Input, hour, minute);
		}

		public Indicators.SpeculatorSethFree.spsfLineAtTime spsfLineAtTime(ISeries<double> input , int hour, int minute)
		{
			return indicator.spsfLineAtTime(input, hour, minute);
		}
	}
}

#endregion
