#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	public class spsfPercentChangeChart : Indicator
	{
		public enum SpreadBarWidthTypeEnum {Auto,	UsePlotWidth}
		
		private string	versionString = "v 1.0  -  Nov 29, 2022";
		
		double start;
		
		SharpDX.Direct2D1.Brush _upBrush;
		SharpDX.Direct2D1.Brush _dnBrush;
		SharpDX.Direct2D1.Brush _naBrush;
		SharpDX.Direct2D1.Brush	_dnOutlineBrush;
		SharpDX.Direct2D1.Brush _upOutlineBrush;
		SharpDX.Direct2D1.Brush _naOutlineBrush;
		
		private Series<int> deltaColorSeries;
		private Series<double>	deltaOpenSeries;
		private Series<double>	deltaCloseSeries;
		private Series<double>	deltaHighSeries;
		private Series<double>	deltaLowSeries;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "PercentChangeChart";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				AddPlot(Brushes.SlateBlue, "change");
				
				BarUpBrush                  = Brushes.CornflowerBlue;
                OutlineUpBrush              = Brushes.CornflowerBlue;
                BarDnBrush                  = Brushes.Orange;
                OutlineDnBrush              = Brushes.Orange;
			}
			else if (State == State.Configure)
			{
			}
			
			if (State == State.DataLoaded)
			{			
				deltaColorSeries = new Series<int>(this, MaximumBarsLookBack.Infinite);
                deltaOpenSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
				deltaCloseSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
				deltaHighSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
				deltaLowSeries = new Series<double>(this, MaximumBarsLookBack.Infinite);
			}
		}
		
		public override void OnRenderTargetChanged()
		{
			// if dxBrush exists on first render target change, dispose of it
		  	if (_upBrush != null) _upBrush.Dispose();			
		  	if (_dnBrush != null) _dnBrush.Dispose();			
		  	if (_naBrush != null) _naBrush.Dispose();
			if (_upOutlineBrush != null) _upOutlineBrush.Dispose();			
		  	if (_dnOutlineBrush != null) _dnOutlineBrush.Dispose();	 
		  	if (_naOutlineBrush != null) _naOutlineBrush.Dispose();
			
			// recalculate dxBrush from value calculated in OnBarUpdated when RenderTarget is recreated
			if (RenderTarget != null)
		  	{
				_upBrush = BarUpBrush.ToDxBrush(RenderTarget);
				_upOutlineBrush = OutlineUpBrush.ToDxBrush(RenderTarget);
				_dnBrush = BarDnBrush.ToDxBrush(RenderTarget);
				_dnOutlineBrush = OutlineDnBrush.ToDxBrush(RenderTarget);
			}
		}

		protected override void OnBarUpdate()
		{
			deltaOpenSeries[0] = Open[0];
			deltaHighSeries[0] = High[0];
			deltaLowSeries[0] = Low[0];
			deltaCloseSeries[0] = Close[0];
			
			if(deltaCloseSeries[0] > deltaOpenSeries[0])
			{
				deltaColorSeries[0] = 1;
			}			
			if(deltaCloseSeries[0] < deltaOpenSeries[0])
			{
				deltaColorSeries[0] = -1;
			}					
		}
		
		protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{	
			start = Open.GetValueAt(ChartBars.FromIndex);
			Print(start);

	        if (Bars == null || ChartControl == null || Bars.Instrument == null || !IsVisible)
	        {
	            return;
	        }

	        try
	        {
				for(int i = ChartBars.FromIndex; i <= ChartBars.ToIndex; i++)
				{
					DrawCandle(chartControl, chartScale, i);
				}
			}
            catch (Exception ex)
            {
                Print("SpreadCandles exception at OnRender()" + ex + ex.InnerException);
            }
		}

		protected void DrawCandle(ChartControl chartControl, ChartScale chartScale, int idx)
		{
			float width = SpreadBarWidthType == SpreadBarWidthTypeEnum.Auto ? (float) Math.Max(2, ChartBars.Properties.ChartStyle.BarWidth*2) : Plots[1].Width;

			double barOpen = (deltaOpenSeries.GetValueAt(idx) - start) / start;
			double barHigh = (deltaHighSeries.GetValueAt(idx) - start) / start;
			double barLow = (deltaLowSeries.GetValueAt(idx) - start) / start;
			double barClose = (deltaCloseSeries.GetValueAt(idx) - start) / start;

			float yOpen = chartScale.GetYByValue(barOpen);
			float yHi = chartScale.GetYByValue(barHigh);
			float yLo = chartScale.GetYByValue(barLow);
			float yClose = chartScale.GetYByValue(barClose);

			var height = (int) Math.Abs(yOpen - yClose);
			var top = (barOpen > barClose ? yOpen : yClose);
			var bottom = (barOpen < barClose ? yOpen : yClose);
			float xCenter = chartControl.GetXByBarIndex(ChartBars, idx);
			float xPosition = xCenter - (float) (width * 0.5f);

			SharpDX.RectangleF rect = new SharpDX.RectangleF(xPosition, top, width, height);

			SharpDX.Direct2D1.Brush brushBarDX = _naBrush;
			SharpDX.Direct2D1.Brush brushOutlineDX = _naBrush;

			if (deltaColorSeries.GetValueAt(idx) ==  1)
			{
				brushBarDX = _upBrush;
				brushOutlineDX = _upOutlineBrush;
			}
			if (deltaColorSeries.GetValueAt(idx) == -1)
			{
				brushBarDX = _dnBrush;
				brushOutlineDX = _dnOutlineBrush;
			}

			RenderTarget.FillRectangle(rect, brushBarDX);
			RenderTarget.DrawRectangle(rect, brushOutlineDX);
			RenderTarget.DrawLine(new SharpDX.Vector2(xCenter, yHi), new SharpDX.Vector2(xCenter, top), brushOutlineDX, 1f);
			RenderTarget.DrawLine(new SharpDX.Vector2(xCenter, yLo), new SharpDX.Vector2(xCenter, bottom), brushOutlineDX, 1f);
		}
		
		
		[XmlIgnore]
	    [Display(ResourceType = typeof(Custom.Resource), Name = "Up color", Order = 6, GroupName = "Display")]
	    public Brush BarUpBrush { get; set; }

	    [Browsable(false)]
	    public string BarUpBrushSerialize
	    {
	        get { return Serialize.BrushToString(BarUpBrush); }
	        set { BarUpBrush = Serialize.StringToBrush(value); }
	    }

	    [XmlIgnore]
	    [Display(ResourceType = typeof(Custom.Resource), Name = "Down color", Order = 8, GroupName = "Display")]
	    public Brush BarDnBrush { get; set; }

	    [Browsable(false)]
	    public string BarDnBrushSerialize
	    {
	        get { return Serialize.BrushToString(BarDnBrush); }
	        set { BarDnBrush = Serialize.StringToBrush(value); }
	    }

	    [XmlIgnore]
	    [Display(ResourceType = typeof(Custom.Resource), Name = "Up color outline", Order = 7, GroupName = "Display")]
	    public Brush OutlineUpBrush { get; set; }

	    [Browsable(false)]
	    public string OutlineUpBrushSerialize
	    {
	        get { return Serialize.BrushToString(OutlineUpBrush); }
	        set { OutlineUpBrush = Serialize.StringToBrush(value); }
	    }

	    [XmlIgnore]
	    [Display(ResourceType = typeof(Custom.Resource), Name = "Dn color outline", Order = 9, GroupName = "Display")]
	    public Brush OutlineDnBrush { get; set; }

        [Browsable(false)]
	    public string OutlineDnBrushSerialize
	    {
	        get { return Serialize.BrushToString(OutlineDnBrush); }
	        set { OutlineDnBrush = Serialize.StringToBrush(value); }
	    }
		
        [Display(Name = "Bar plot width", Description = "Choose between auto or fixed width. For fixed width use the plot width to adjust.", Order = 0, GroupName = "Display")]
        public SpreadBarWidthTypeEnum SpreadBarWidthType { get; set; }	

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> Spread
		{
			get { return Values[0]; }
		}

		[XmlIgnore]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Release and date", Description = "Release and date", GroupName = "Version", Order = 0)]
		public string VersionString
		{	
            get { return versionString; }
            set { ; }
		}
		
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfPercentChangeChart[] cachespsfPercentChangeChart;
		public SpeculatorSethFree.spsfPercentChangeChart spsfPercentChangeChart()
		{
			return spsfPercentChangeChart(Input);
		}

		public SpeculatorSethFree.spsfPercentChangeChart spsfPercentChangeChart(ISeries<double> input)
		{
			if (cachespsfPercentChangeChart != null)
				for (int idx = 0; idx < cachespsfPercentChangeChart.Length; idx++)
					if (cachespsfPercentChangeChart[idx] != null &&  cachespsfPercentChangeChart[idx].EqualsInput(input))
						return cachespsfPercentChangeChart[idx];
			return CacheIndicator<SpeculatorSethFree.spsfPercentChangeChart>(new SpeculatorSethFree.spsfPercentChangeChart(), input, ref cachespsfPercentChangeChart);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfPercentChangeChart spsfPercentChangeChart()
		{
			return indicator.spsfPercentChangeChart(Input);
		}

		public Indicators.SpeculatorSethFree.spsfPercentChangeChart spsfPercentChangeChart(ISeries<double> input )
		{
			return indicator.spsfPercentChangeChart(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfPercentChangeChart spsfPercentChangeChart()
		{
			return indicator.spsfPercentChangeChart(Input);
		}

		public Indicators.SpeculatorSethFree.spsfPercentChangeChart spsfPercentChangeChart(ISeries<double> input )
		{
			return indicator.spsfPercentChangeChart(input);
		}
	}
}

#endregion
