#region Using declarations
using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	public class spsfPriorDayVPLevels : Indicator
	{
		public class RowData
		{
			public long TotalVolume;
			public double Price;
			
			public RowData(long totalVolume, double price)
			{
				TotalVolume = totalVolume;
				Price = price;				
			}
		}		
		
		public class Profile
		{
			public Dictionary<double, RowData> Data;
			public long PocValue = 0;
			public double PocPrice = 0;
			public long ProfileTotalVolume = 0;
			public double TickSize;
			public double Percent;

			public Profile(double tickSize, double percent)
			{
				TickSize = tickSize;
				Percent = percent;
				
				Data = new Dictionary<double, RowData>();
				ProfileTotalVolume = 0;
			}
			
			public void AddVolume(double price, long volume)
			{
				ProfileTotalVolume += volume;
				
				RowData val;
				
				if (Data.TryGetValue(price, out val))
			    {
					var total = val.TotalVolume + volume;
					val.TotalVolume = total;	
					
					if(total > PocValue)
					{
						PocValue = total;
						PocPrice = price;
					}
			    }
			    else
			    {
			        Data.Add(price, new RowData(volume, price));
					
					if(volume > PocValue)
					{
						PocValue = volume;
						PocPrice = price;
					}
			    }		
			}
			
			public double[] CalcValueArea()
			{
				double vah = PocPrice + TickSize;
				double val = PocPrice - TickSize;
				
				long vav = PocValue;
				
				while ( vav < ProfileTotalVolume * Percent)
				{			
					if (Data.ContainsKey(vah))
					{
						vav += Data[vah].TotalVolume;
						vah += TickSize;
					}
					if (Data.ContainsKey(val))
					{
						vav += Data[val].TotalVolume;
						val -= TickSize;
					}					
				}
				
				return new double[]{vah, val};
			}
			
		}		
		
		private SessionIterator sessionIterator;
		private Profile myProfile;
		private bool firstDay = true;
		
		private Dictionary<DateTime, Double[]> _cache = new Dictionary<DateTime, Double[]>();
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"The point of control or price with most volume for that bar.";
				Name										= "PriorDayVPLevels";
				Calculate									= Calculate.OnEachTick;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= false;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= false;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= false;
				
				Percent = .7;
				
				CachePath = "ProfileLevels.csv";
				
				AddPlot(new Stroke(Brushes.Yellow, DashStyleHelper.Solid, 3), PlotStyle.Hash, "POC");
				AddPlot(new Stroke(Brushes.DodgerBlue, DashStyleHelper.Solid, 3), PlotStyle.Hash, "VAH");
				AddPlot(new Stroke(Brushes.Red, DashStyleHelper.Solid, 3), PlotStyle.Hash, "VAL");
			}
			else if (State == State.Configure)
			{
				sessionIterator = new SessionIterator(Bars);
			}
			else if (State == State.DataLoaded)
			{
				Import();
				 myProfile = new Profile(TickSize, Percent);	
			}
		}
		
		protected override void OnMarketData(MarketDataEventArgs e)
		{
			if(CurrentBar < 0)
				return;
			
			//DateTime tomorrow = sessionIterator..GetNextSession(Time[0], false);
			DateTime current = sessionIterator.GetTradingDay(Time[0]).Date;
			
			if(HasData(current))
				return;
			
			if (e.MarketDataType == MarketDataType.Last)
			{
				myProfile.AddVolume(e.Price, e.Volume);	
			}
		}

		protected override void OnBarUpdate()
		{		
			if (!IsFirstTickOfBar || CurrentBar == 0) return;
			
			DateTime previous = sessionIterator.GetTradingDay(Time[1]);
			DateTime current = sessionIterator.GetTradingDay(Time[0]);
			
			if (current.DayOfWeek.Equals(previous.DayOfWeek))
			{
				POC[0] = POC[1];
				VAH[0] = VAH[1];
				VAL[0] = VAL[1];
			}
			else
			{							
				DateTime date = previous.Date;
				
				if(HasData(date))
				{
					double[] data = _cache[date];
					POC[0] = data[0];
					VAH[0] = data[1];
					VAL[0] = data[2];
				}
				else
				{			
					NewDay(previous, current);
				}
			}
		}
		
		protected void NewDay(DateTime previous, DateTime current)
		{
			double[] valuearea = myProfile.CalcValueArea();
			
			VAH[0] = valuearea[0];
			VAL[0] = valuearea[1];
			POC[0] = myProfile.PocPrice;
			
			DateTime date = previous.Date;
			
			Export(date);
			double [] data = new double []{POC[0], VAH[0], VAL[0]};
			_cache.Add(date, data);
			
			myProfile = new Profile(TickSize, Percent);			
		}
		
		protected void Export(DateTime dateTime)
		{
			StreamWriter writetext = null;
			
			try
			{
				writetext = new StreamWriter(NinjaTrader.Core.Globals.UserDataDir + "cache\\" + CachePath, true);
				string date = dateTime.ToString();
				string vah = VAH[0].ToString();
				string val = VAL[0].ToString();
				string poc = POC[0].ToString();
				string data = string.Format("{0}, {1}, {2}, {3}", date, poc, vah, val);
				
				writetext.WriteLine(data);	
				Print("Saved: " + dateTime.ToString());
			}
			catch(Exception e)
			{
				Print("Error exporting data to file " + e.Message);
			}
			finally
			{
				if(writetext != null)
					writetext.Dispose();
			}
		}
		
		protected void Import()
		{
			
			string folder = NinjaTrader.Core.Globals.UserDataDir + "cache\\";
			System.IO.Directory.CreateDirectory(folder);
			
			try{
				using(StreamReader sr = new StreamReader(folder + CachePath))
				{
					string line;
					
					while ((line = sr.ReadLine()) != null) 
	                {
						int comma = line.IndexOf(',');
						DateTime entrydate = DateTime.Parse(line.Substring(0, comma));					
						
						string doubles = line.Substring(comma+1);						
						double[] data = Array.ConvertAll(doubles.Split(','), Double.Parse);
						Double[] duplicate;
						
						_cache.Add(entrydate, data);
					}
				}
			}
			catch(FileNotFoundException e)
			{
				Print("Cache file not found: " + folder + CachePath);
			}
		}
		
		protected bool HasData(DateTime date)
		{
			return _cache.ContainsKey(date);
		}
					
		
		[NinjaScriptProperty]
		public double Percent
		{get; set;}		
				
		[NinjaScriptProperty]
		public String CachePath
		{ get; set; }
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> POC
		{
			get { return Values[0]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> VAH
		{
			get { return Values[1]; }
		}
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> VAL
		{
			get { return Values[2]; }
		}
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfPriorDayVPLevels[] cachespsfPriorDayVPLevels;
		public SpeculatorSethFree.spsfPriorDayVPLevels spsfPriorDayVPLevels(double percent, String cachePath)
		{
			return spsfPriorDayVPLevels(Input, percent, cachePath);
		}

		public SpeculatorSethFree.spsfPriorDayVPLevels spsfPriorDayVPLevels(ISeries<double> input, double percent, String cachePath)
		{
			if (cachespsfPriorDayVPLevels != null)
				for (int idx = 0; idx < cachespsfPriorDayVPLevels.Length; idx++)
					if (cachespsfPriorDayVPLevels[idx] != null && cachespsfPriorDayVPLevels[idx].Percent == percent && cachespsfPriorDayVPLevels[idx].CachePath == cachePath && cachespsfPriorDayVPLevels[idx].EqualsInput(input))
						return cachespsfPriorDayVPLevels[idx];
			return CacheIndicator<SpeculatorSethFree.spsfPriorDayVPLevels>(new SpeculatorSethFree.spsfPriorDayVPLevels(){ Percent = percent, CachePath = cachePath }, input, ref cachespsfPriorDayVPLevels);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfPriorDayVPLevels spsfPriorDayVPLevels(double percent, String cachePath)
		{
			return indicator.spsfPriorDayVPLevels(Input, percent, cachePath);
		}

		public Indicators.SpeculatorSethFree.spsfPriorDayVPLevels spsfPriorDayVPLevels(ISeries<double> input , double percent, String cachePath)
		{
			return indicator.spsfPriorDayVPLevels(input, percent, cachePath);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfPriorDayVPLevels spsfPriorDayVPLevels(double percent, String cachePath)
		{
			return indicator.spsfPriorDayVPLevels(Input, percent, cachePath);
		}

		public Indicators.SpeculatorSethFree.spsfPriorDayVPLevels spsfPriorDayVPLevels(ISeries<double> input , double percent, String cachePath)
		{
			return indicator.spsfPriorDayVPLevels(input, percent, cachePath);
		}
	}
}

#endregion
