#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Core;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
using SharpDX.DirectWrite;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	[Gui.CategoryOrder("Parameters", 1)]
	[Gui.CategoryOrder("Display", 15)]
	[Gui.CategoryOrder("Data Series", 20)]
	[Gui.CategoryOrder("Setup", 30)]
	[Gui.CategoryOrder("Visual", 40)]
	[Gui.CategoryOrder("Plots", 50)]
	[Gui.CategoryOrder("Version", 80)]	
	public class spsfRangeHistogramFree : Indicator
	{
		private string	versionString = "v 1.0  -  Sept 15, 2022";
		[XmlIgnore]
		public Dictionary<double, int> Profile = new Dictionary<double, int>();
		
		public int Max = 0;
		public double MaxId = 0;
		public int TotalSamples = 0;
		public int ValueArea = 1;
		
		protected bool setScale = false;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"A histogram showing the distribution and standard distribution of bar ranges.";
				Name										= "RangeHistogramFree";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				BarsRequiredToPlot			= 0;
				
				ValueAreaPercent = .7;
				NormalColor  = Brushes.White;
				ValueColor = Brushes.CornflowerBlue;
				
				textSize			= 11;
				TextColor = Brushes.Black;
			}
			else if (State == State.Historical)
			{
				if (ChartControl == null) return;
				
				InitializeLifetimeDrawingTools();
			}
			else if (State == State.Configure)
			{
				XmlSerializer s = new XmlSerializer(typeof(spsfRangeHistogramFree));
			}
		}

		protected override void OnBarUpdate()
		{
			
			double rangeInTicks = Range()[0] / TickSize;
			
			if (Profile.ContainsKey(rangeInTicks))
		    {
                Profile[rangeInTicks]++;
		    }
		    else
		    {
		        Profile.Add(rangeInTicks, 1);
				setScale = true;
		    }	

			TotalSamples++;
			
			CalcSamples();
		}
	
		protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{	
			if(Profile.Count < 1)
				return;
			
			base.OnRender(chartControl, chartScale);				
			if(Bars == null || Bars.Instrument == null || IsInHitTest || CurrentBar < 1) { return; }
			
			
			try
			{

				
				foreach(KeyValuePair<double, int> row in Profile)
				{
					drawRow(chartControl, chartScale, row.Key, row.Value);
				}
			}
			catch(Exception e)
			{
				Print("Error occured during drawing");
			}
		}
		
		
		private void CalcSamples()
		{
			KeyValuePair<double, int> maxkvp = Profile.First();
			foreach(KeyValuePair<double, int> row in Profile)
			{
				if(row.Value > maxkvp.Value)
				{
					maxkvp = row;
				}
			}
			
			Max = maxkvp.Value;
			MaxId = maxkvp.Key;
			
			int i;
			int valueArea = Max;
			for(i = 1; i < Profile.Count; i++)	
			{
				if(Profile.ContainsKey(MaxId + i))
				{
					valueArea = valueArea + Profile[MaxId + i];
				}
				
				if(Profile.ContainsKey(MaxId - i))
				{
					valueArea = valueArea + Profile[MaxId - i];
				}
				
				if(valueArea > TotalSamples * ValueAreaPercent)
				{
					ValueArea = i;
					break;
				}
			}
		}
		
		private void drawRow(ChartControl chartControl, ChartScale chartScale, double value, int quantity)
		{
			//Calculate color of this row.
			//Brush brushColor	= new SolidColorBrush(System.Windows.Media.Color.FromRgb(255, 0, 0)); //bidColor.Freeze();		
			//float alpha = alpha = (float)((double)Math.Abs(row.TotalVolume) / (double)myProfile.HiValue);
			Brush brushColor = NormalColor;	
			if(value <= MaxId + ValueArea && value >= MaxId - ValueArea)
			{				
				brushColor = ValueColor;
			}
			
				
			//Calculate cell properties
			double y1 = ((chartScale.GetYByValue(value) + chartScale.GetYByValue(value + 1)) / 2) + 1;
			double y2 = ((chartScale.GetYByValue(value) + chartScale.GetYByValue(value - 1)) / 2) - 1;
			
			SharpDX.RectangleF rect = new SharpDX.RectangleF();
			rect.X      = (float)chartControl.CanvasRight;
			rect.Y      = (float)y1;
					
			rect.Width  = (float)((chartControl.CanvasLeft - chartControl.CanvasRight) * Math.Log(quantity) / Math.Log(Max));			
			rect.Height = (float)Math.Abs(y1 - y2);			

			//Draw the row.
			using(SharpDX.Direct2D1.Brush rowBrush =  brushColor.ToDxBrush(RenderTarget))
			{
				RenderTarget.FillRectangle(rect, rowBrush);
				//RenderTarget.FillRectangle(rect, rowBrush);
			}
			
//			if(rect.Height > this.MinimumTextHeight)
//			{
//				RenderTarget.DrawText(string.Format("{0}", quantity), textFormat, rect, TextColor.ToDxBrush(RenderTarget));
//			}
		}
		
		public override void OnCalculateMinMax()
		{
			try
			{
				if(Profile.Count < 1)
					return;
				
				MinValue = Profile.Keys.Min();
				MaxValue = Profile.Keys.Max();
			}
			catch(Exception e)
			{
				Print("Error occured calculating min/max");
			}
		}
		
		public int CalculateMinimumTextHeight()
		{
			return getTextHeight("0") - 5;
		}
		
		private int getTextHeight(string text)
		{
			SimpleFont sf = new NinjaTrader.Gui.Tools.SimpleFont("Consolas", textSize);
			
			float textHeight = 0f;
			
			if(text.Length > 0)
			{
				TextFormat tf = new TextFormat(new SharpDX.DirectWrite.Factory(), sf.Family.ToString(), SharpDX.DirectWrite.FontWeight.Normal, SharpDX.DirectWrite.FontStyle.Normal, (float)sf.Size);
				TextLayout tl = new TextLayout(Core.Globals.DirectWriteFactory, text, tf, ChartPanel.W, ChartPanel.H);
				
				textHeight = tl.Metrics.Height;
				
				tf.Dispose();
				tl.Dispose();
			}
			
			return (int)textHeight;
		}
		
		public void InitializeLifetimeDrawingTools()
		{		
			textFormat = new TextFormat(Globals.DirectWriteFactory, "Arial", FontWeight.Bold, FontStyle.Normal, FontStretch.Normal, textSize)
            {
                TextAlignment = TextAlignment.Trailing,   //TextAlignment.Leading,
                WordWrapping = WordWrapping.NoWrap
            };
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(0, 1)]
		[Display(Name="Value Area Percent", Description="Percent of values to show within value area.", Order=1, GroupName="Parameters")]
		public double ValueAreaPercent
		{ get; set; }
		
		[NinjaScriptProperty]
		[XmlIgnore]
		[Display(Name = "Normal values color", GroupName = "Parameters", Order = 9)]
		public Brush NormalColor
		{ get; set; }
		
		[Browsable(false)]
		public string normalColorSerializable
		{
			get { return Serialize.BrushToString(NormalColor); }
			set { NormalColor = Serialize.StringToBrush(value); }
		}
		
		[NinjaScriptProperty]
		[XmlIgnore]
		[Display(Name = "Value area Color", GroupName = "Parameters", Order = 9)]
		public Brush ValueColor
		{ get; set; }
		
		[Browsable(false)]
		public string valueColorSerializable
		{
			get { return Serialize.BrushToString(ValueColor); }
			set { ValueColor = Serialize.StringToBrush(value); }
		}
		
		protected TextFormat textFormat
		{ get; set; }
		
		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(Name = "Text Size", GroupName = "Parameters", Order = 7)]
		public int textSize
		{ get; set; }
		
		[NinjaScriptProperty]
		[XmlIgnore]
		[Display(Name = "Text Color", GroupName = "Parameters", Order = 9)]
		public Brush TextColor
		{ get; set; }
		
		[Browsable(false)]
		public string TextColorSerializable
		{
			get { return Serialize.BrushToString(TextColor); }
			set { TextColor = Serialize.StringToBrush(value); }
		}
		
		protected int MinimumTextHeight
		{ get; set; }


		[XmlIgnore]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Release and date", Description = "Release and date", GroupName = "Version", Order = 0)]
		public string VersionString
		{	
            get { return versionString; }
            set { ; }
		}	
		#endregion		
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfRangeHistogramFree[] cachespsfRangeHistogramFree;
		public SpeculatorSethFree.spsfRangeHistogramFree spsfRangeHistogramFree(double valueAreaPercent, Brush normalColor, Brush valueColor, int textSize, Brush textColor)
		{
			return spsfRangeHistogramFree(Input, valueAreaPercent, normalColor, valueColor, textSize, textColor);
		}

		public SpeculatorSethFree.spsfRangeHistogramFree spsfRangeHistogramFree(ISeries<double> input, double valueAreaPercent, Brush normalColor, Brush valueColor, int textSize, Brush textColor)
		{
			if (cachespsfRangeHistogramFree != null)
				for (int idx = 0; idx < cachespsfRangeHistogramFree.Length; idx++)
					if (cachespsfRangeHistogramFree[idx] != null && cachespsfRangeHistogramFree[idx].ValueAreaPercent == valueAreaPercent && cachespsfRangeHistogramFree[idx].NormalColor == normalColor && cachespsfRangeHistogramFree[idx].ValueColor == valueColor && cachespsfRangeHistogramFree[idx].textSize == textSize && cachespsfRangeHistogramFree[idx].TextColor == textColor && cachespsfRangeHistogramFree[idx].EqualsInput(input))
						return cachespsfRangeHistogramFree[idx];
			return CacheIndicator<SpeculatorSethFree.spsfRangeHistogramFree>(new SpeculatorSethFree.spsfRangeHistogramFree(){ ValueAreaPercent = valueAreaPercent, NormalColor = normalColor, ValueColor = valueColor, textSize = textSize, TextColor = textColor }, input, ref cachespsfRangeHistogramFree);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfRangeHistogramFree spsfRangeHistogramFree(double valueAreaPercent, Brush normalColor, Brush valueColor, int textSize, Brush textColor)
		{
			return indicator.spsfRangeHistogramFree(Input, valueAreaPercent, normalColor, valueColor, textSize, textColor);
		}

		public Indicators.SpeculatorSethFree.spsfRangeHistogramFree spsfRangeHistogramFree(ISeries<double> input , double valueAreaPercent, Brush normalColor, Brush valueColor, int textSize, Brush textColor)
		{
			return indicator.spsfRangeHistogramFree(input, valueAreaPercent, normalColor, valueColor, textSize, textColor);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfRangeHistogramFree spsfRangeHistogramFree(double valueAreaPercent, Brush normalColor, Brush valueColor, int textSize, Brush textColor)
		{
			return indicator.spsfRangeHistogramFree(Input, valueAreaPercent, normalColor, valueColor, textSize, textColor);
		}

		public Indicators.SpeculatorSethFree.spsfRangeHistogramFree spsfRangeHistogramFree(ISeries<double> input , double valueAreaPercent, Brush normalColor, Brush valueColor, int textSize, Brush textColor)
		{
			return indicator.spsfRangeHistogramFree(input, valueAreaPercent, normalColor, valueColor, textSize, textColor);
		}
	}
}

#endregion
