#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	[Gui.CategoryOrder("Parameters", 1)]
	[Gui.CategoryOrder("Data Series", 20)]
	[Gui.CategoryOrder("Setup", 30)]
	[Gui.CategoryOrder("Visual", 40)]
	[Gui.CategoryOrder("Plots", 50)]
	[Gui.CategoryOrder("Version", 80)]	
	public class spsfSpreadFree : Indicator
	{
		private string	versionString = "v 1.0  -  April 18, 2020";
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Shows the difference between two instruments.";
				Name										= "Spread Free";
				Calculate									= Calculate.OnEachTick;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				BarsRequiredToPlot							= 20;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				FrontLeg					= string.Empty;
				BackLeg					= string.Empty;
				FrontRatio 				= 3;
				BackRatio				= 1;
				
				AddPlot(Brushes.Orange, "MySpread");
			}
			else if (State == State.Configure)
			{		
				string MyFrontLeg = FrontLeg;
				string MyBackLeg = BackLeg;					

				AddDataSeries(MyFrontLeg, BarsArray[0].BarsPeriod);
				AddDataSeries(MyBackLeg, BarsArray[0].BarsPeriod);
			}
		}

		protected override void OnBarUpdate()
		{

			if (CurrentBars[0] <= BarsRequiredToPlot || CurrentBars[1] <= BarsRequiredToPlot || CurrentBars[2] <= BarsRequiredToPlot)
        		return;
			
			Value[0] = BarsArray[1][0] * FrontRatio - BarsArray[2][0] * BackRatio;			
		}

		#region Properties
		
		[Browsable(false)]
		[XmlIgnore]
		public Series<double> MySpread
		{
			get { return Values[0]; }
		}
		
		[NinjaScriptProperty]
		[Display(Name="FrontLeg", Description="Front leg of the contract EG ZN 06-17", Order=0, GroupName="Parameters")]
		public string FrontLeg
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="FrontRatio", Description="Ratio for front leg.  See http://www.cmegroup.com/trading/interest-rates/intercommodity-spread.html", Order=1, GroupName="Parameters")]
		public int FrontRatio
		{ get; set; }

		[NinjaScriptProperty]
		[Display(Name="BackLeg", Description="Back leg of the contract eg ZB 06-17", Order=2, GroupName="Parameters")]
		public string BackLeg
		{ get; set; }
				
		[NinjaScriptProperty]
		[Display(Name="BackRatio", Description="Ratio for back leg.  See http://www.cmegroup.com/trading/interest-rates/intercommodity-spread.html", Order=3, GroupName="Parameters")]
		public int BackRatio
		{ get; set; }
		
		[XmlIgnore]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Release and date", Description = "Release and date", GroupName = "Version", Order = 0)]
		public string VersionString
		{	
            get { return versionString; }
            set { ; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfSpreadFree[] cachespsfSpreadFree;
		public SpeculatorSethFree.spsfSpreadFree spsfSpreadFree(string frontLeg, int frontRatio, string backLeg, int backRatio)
		{
			return spsfSpreadFree(Input, frontLeg, frontRatio, backLeg, backRatio);
		}

		public SpeculatorSethFree.spsfSpreadFree spsfSpreadFree(ISeries<double> input, string frontLeg, int frontRatio, string backLeg, int backRatio)
		{
			if (cachespsfSpreadFree != null)
				for (int idx = 0; idx < cachespsfSpreadFree.Length; idx++)
					if (cachespsfSpreadFree[idx] != null && cachespsfSpreadFree[idx].FrontLeg == frontLeg && cachespsfSpreadFree[idx].FrontRatio == frontRatio && cachespsfSpreadFree[idx].BackLeg == backLeg && cachespsfSpreadFree[idx].BackRatio == backRatio && cachespsfSpreadFree[idx].EqualsInput(input))
						return cachespsfSpreadFree[idx];
			return CacheIndicator<SpeculatorSethFree.spsfSpreadFree>(new SpeculatorSethFree.spsfSpreadFree(){ FrontLeg = frontLeg, FrontRatio = frontRatio, BackLeg = backLeg, BackRatio = backRatio }, input, ref cachespsfSpreadFree);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfSpreadFree spsfSpreadFree(string frontLeg, int frontRatio, string backLeg, int backRatio)
		{
			return indicator.spsfSpreadFree(Input, frontLeg, frontRatio, backLeg, backRatio);
		}

		public Indicators.SpeculatorSethFree.spsfSpreadFree spsfSpreadFree(ISeries<double> input , string frontLeg, int frontRatio, string backLeg, int backRatio)
		{
			return indicator.spsfSpreadFree(input, frontLeg, frontRatio, backLeg, backRatio);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfSpreadFree spsfSpreadFree(string frontLeg, int frontRatio, string backLeg, int backRatio)
		{
			return indicator.spsfSpreadFree(Input, frontLeg, frontRatio, backLeg, backRatio);
		}

		public Indicators.SpeculatorSethFree.spsfSpreadFree spsfSpreadFree(ISeries<double> input , string frontLeg, int frontRatio, string backLeg, int backRatio)
		{
			return indicator.spsfSpreadFree(input, frontLeg, frontRatio, backLeg, backRatio);
		}
	}
}

#endregion
