#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	public class spsfVWAPTypical : Indicator
	{
				double TotalVolume = 0;
		double TotalValue = 0;
		
		int FirstBar = 0;
		private SessionIterator sessionIterator;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"VWAP cacluated using typical price to avoid loading extra data series underneath.";
				Name										= "VWAPTypical";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				BarsRequiredToPlot							= 0;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				AddPlot(Brushes.LightCyan, "VWAP");
			}
			else if (State == State.Configure)
			{
				sessionIterator = new SessionIterator(Bars);
			}
		}

		protected override void OnBarUpdate()
		{
			if(CurrentBars[0] <= BarsRequiredToPlot) return;

			//Reset profile on session week or day.
			if(IsFirstTickOfBar)
			{
				DateTime previous = sessionIterator.GetTradingDay(Time[1]);
				DateTime current = sessionIterator.GetTradingDay(Time[0]);
				
				//Reset profile on daily basis.
				if(Bars.IsFirstBarOfSession)
				{			
					FirstBar = CurrentBar;
					Reset();
				}
			}
			
			
			double typicalPrice = (High[0] + Low[0] + Close[0]) / 3.0;
			double typicalValue = typicalPrice * Volume[0];

			TotalValue += typicalValue;
			TotalVolume += Volume[0];
			
			if(TotalVolume == 0)
				VWAP[0] = Close[0];
			else
				VWAP[0] = TotalValue / TotalVolume;
		}
		
		protected void Reset()
		{
			TotalVolume = 0;
			TotalValue = 0;
		}	
		
		#region Properties		

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> VWAP
		{
			get { return Values[0]; }
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfVWAPTypical[] cachespsfVWAPTypical;
		public SpeculatorSethFree.spsfVWAPTypical spsfVWAPTypical()
		{
			return spsfVWAPTypical(Input);
		}

		public SpeculatorSethFree.spsfVWAPTypical spsfVWAPTypical(ISeries<double> input)
		{
			if (cachespsfVWAPTypical != null)
				for (int idx = 0; idx < cachespsfVWAPTypical.Length; idx++)
					if (cachespsfVWAPTypical[idx] != null &&  cachespsfVWAPTypical[idx].EqualsInput(input))
						return cachespsfVWAPTypical[idx];
			return CacheIndicator<SpeculatorSethFree.spsfVWAPTypical>(new SpeculatorSethFree.spsfVWAPTypical(), input, ref cachespsfVWAPTypical);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfVWAPTypical spsfVWAPTypical()
		{
			return indicator.spsfVWAPTypical(Input);
		}

		public Indicators.SpeculatorSethFree.spsfVWAPTypical spsfVWAPTypical(ISeries<double> input )
		{
			return indicator.spsfVWAPTypical(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfVWAPTypical spsfVWAPTypical()
		{
			return indicator.spsfVWAPTypical(Input);
		}

		public Indicators.SpeculatorSethFree.spsfVWAPTypical spsfVWAPTypical(ISeries<double> input )
		{
			return indicator.spsfVWAPTypical(input);
		}
	}
}

#endregion
