//+----------------------------------------------------------------------------------------------+
//| Copyright © <2022>  <SpeculatorSeth.com>
//
//| This program is free software: you can redistribute it and/or modify
//| it under the terms of the GNU General Public License as published by
//| the Free Software Foundation, either version 3 of the License, or
//| any later version.
//|
//| This program is distributed in the hope that it will be useful,
//| but WITHOUT ANY WARRANTY; without even the implied warranty of
//| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//| GNU General Public License for more details.
//|
//| By installing this software you confirm acceptance of the GNU
//| General Public License terms. You may find a copy of the license
//| here; http://www.gnu.org/licenses/
//+----------------------------------------------------------------------------------------------+

#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	[Gui.CategoryOrder("Parameters", 1)]
	[Gui.CategoryOrder("Display", 15)]
	[Gui.CategoryOrder("Data Series", 20)]
	[Gui.CategoryOrder("Setup", 30)]
	[Gui.CategoryOrder("Visual", 40)]
	[Gui.CategoryOrder("Plots", 50)]
	[Gui.CategoryOrder("Version", 80)]	
	public class spsfWatermark : Indicator
	{
		private string	versionString = "v 1.0  -  Sept 15, 2022";		
		private WaterMarkType		waterMarkType	= WaterMarkType.InstrumentSymbol;				
		private string 				myWaterMarkText;
		
		
		SharpDX.Vector2 startPoint;
		SharpDX.DirectWrite.TextFormat textFormat;
		SharpDX.DirectWrite.TextLayout textLayout;
		SharpDX.RectangleF rectangleF;
		SharpDX.Direct2D1.Brush customDXBrush;
		SharpDX.Vector2 _v;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Add Watermarks to a chart";
				Name										= "Watermark";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= true;
				DisplayInDataBox							= false;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= false;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				WaterMarkSize					= 200;
				WaterMarkOpacity				= 0.2f;
				TextColor					= Brushes.White;
				CustomText						= "Sample Text";
			}
			if (State == State.Historical)
			{
			    // Make sure our object plots behind the chart bars
				SetZOrder(-1);
			}
			if (State == State.Configure)
			{
				if (waterMarkType == WaterMarkType.CustomText)
				{
					myWaterMarkText = CustomText;
				}
				else if (waterMarkType == WaterMarkType.InstrumentDescription)
				{
					myWaterMarkText = BarsArray[0].Instrument.MasterInstrument.Description;
				}
				else if (waterMarkType == WaterMarkType.InstrumentSymbolAndExpiry)
				{
					myWaterMarkText = BarsArray[0].Instrument.FullName;
				}
				else
				{
					myWaterMarkText = BarsArray[0].Instrument.MasterInstrument.Name;
				}
			}
			else if (State == State.DataLoaded)
			{			
				
				
				SetZOrder(-1);

				Name = "";
				UpdateLayout();
			}
		}
		
		protected override void OnRender(ChartControl cc, ChartScale cs)
		{
			if(_v.X <10)
				UpdateLayout();
			
			SetZOrder(-1);	
			if(textLayout != null && customDXBrush != null)
			{
				RenderTarget.DrawTextLayout(_v, textLayout, customDXBrush);
			}
		}
		
		public override void OnRenderTargetChanged()
		{
			// if dxBrush exists on first render target change, dispose of it
		  	if (customDXBrush != null) customDXBrush.Dispose();			
			if (textFormat != null) textFormat.Dispose();
			if (textLayout != null) textLayout.Dispose();
		 
		  	// recalculate dxBrush from value calculated in OnBarUpdated when RenderTarget is recreated
			if (RenderTarget != null)
		  	{
				customDXBrush = TextColor.ToDxBrush(RenderTarget);
			  	customDXBrush.Opacity = WaterMarkOpacity;
			  	
				UpdateLayout();
			}
		}
		
		private void UpdateLayout()
		{
			if(ChartControl != null && ChartPanel != null)
			{
				textFormat = new SharpDX.DirectWrite.TextFormat(Core.Globals.DirectWriteFactory, ChartControl.Properties.LabelFont.FamilySerialize, WaterMarkSize);				
				textLayout = new SharpDX.DirectWrite.TextLayout(Core.Globals.DirectWriteFactory, myWaterMarkText, textFormat, ChartPanel.W, ChartPanel.H);
				  
				float x = (ChartPanel.W - textLayout.Metrics.Width) / 2;
				float y = (ChartPanel.H - textLayout.Metrics.Height) / 2;
					
				_v = new SharpDX.Vector2(x, y);
			}
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="WaterMarkSize", Order=1, GroupName="Parameters")]
		public int WaterMarkSize
		{ get; set; }

		[NinjaScriptProperty]
		[Range(0, 1)]
		[Display(Name="WaterMarkOpacity", Description="Enter Opacity from 0 to 1", Order=2, GroupName="Parameters")]
		public float WaterMarkOpacity
		{ get; set; }
		
		[XmlIgnore]
		[Display(Name="Text Color", Description="The color of the text", Order=4, GroupName="Parameters")]
		public Brush TextColor
		{ get; set; }

		[Browsable(false)]
		public string TextColorSerializable
		{
			get { return Serialize.BrushToString(TextColor); }
			set { TextColor = Serialize.StringToBrush(value); }
		}	

		[Display(Name="WaterMarkType", Order=4, GroupName = "Parameters", Description="Choose the watermark type")]
		public WaterMarkType WaterMarkType
		{
			get { return waterMarkType; }
			set { waterMarkType = value; }
		}

		[NinjaScriptProperty]
		[Display(Name="CustomText", Description="Leave this blank if you are using stock symbol", Order=5, GroupName="Parameters")]
		public string CustomText
		{ get; set; }

		[XmlIgnore]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Release and date", Description = "Release and date", GroupName = "Version", Order = 0)]
		public string VersionString
		{	
            get { return versionString; }
            set { ; }
		}
		#endregion
	}
}

public enum WaterMarkType
{
	InstrumentSymbol,
	InstrumentSymbolAndExpiry,
	InstrumentDescription,
	CustomText
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfWatermark[] cachespsfWatermark;
		public SpeculatorSethFree.spsfWatermark spsfWatermark(int waterMarkSize, float waterMarkOpacity, string customText)
		{
			return spsfWatermark(Input, waterMarkSize, waterMarkOpacity, customText);
		}

		public SpeculatorSethFree.spsfWatermark spsfWatermark(ISeries<double> input, int waterMarkSize, float waterMarkOpacity, string customText)
		{
			if (cachespsfWatermark != null)
				for (int idx = 0; idx < cachespsfWatermark.Length; idx++)
					if (cachespsfWatermark[idx] != null && cachespsfWatermark[idx].WaterMarkSize == waterMarkSize && cachespsfWatermark[idx].WaterMarkOpacity == waterMarkOpacity && cachespsfWatermark[idx].CustomText == customText && cachespsfWatermark[idx].EqualsInput(input))
						return cachespsfWatermark[idx];
			return CacheIndicator<SpeculatorSethFree.spsfWatermark>(new SpeculatorSethFree.spsfWatermark(){ WaterMarkSize = waterMarkSize, WaterMarkOpacity = waterMarkOpacity, CustomText = customText }, input, ref cachespsfWatermark);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfWatermark spsfWatermark(int waterMarkSize, float waterMarkOpacity, string customText)
		{
			return indicator.spsfWatermark(Input, waterMarkSize, waterMarkOpacity, customText);
		}

		public Indicators.SpeculatorSethFree.spsfWatermark spsfWatermark(ISeries<double> input , int waterMarkSize, float waterMarkOpacity, string customText)
		{
			return indicator.spsfWatermark(input, waterMarkSize, waterMarkOpacity, customText);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfWatermark spsfWatermark(int waterMarkSize, float waterMarkOpacity, string customText)
		{
			return indicator.spsfWatermark(Input, waterMarkSize, waterMarkOpacity, customText);
		}

		public Indicators.SpeculatorSethFree.spsfWatermark spsfWatermark(ISeries<double> input , int waterMarkSize, float waterMarkOpacity, string customText)
		{
			return indicator.spsfWatermark(input, waterMarkSize, waterMarkOpacity, customText);
		}
	}
}

#endregion
