#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//Created by SpeculatorSeth.

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators.SpeculatorSethFree
{
	[Gui.CategoryOrder("Parameters", 1)]
	[Gui.CategoryOrder("Display", 15)]
	[Gui.CategoryOrder("Data Series", 20)]
	[Gui.CategoryOrder("Setup", 30)]
	[Gui.CategoryOrder("Visual", 40)]
	[Gui.CategoryOrder("Plots", 50)]
	[Gui.CategoryOrder("Version", 80)]
	public class spsfYieldCurveFree : Indicator
	{
		private string	versionString = "v 1.0  -  Sept 15, 2022";
		string[] maturities = new string[] {"2YR", "5YR", "10YR", "30YR"};
		
		string name2Year = "2YR";
		string name5Year = "5YR";
		string name10Year = "10YR";
		string name30Year = "30YR";
		
		SharpDX.DirectWrite.TextFormat textFormat;
		SharpDX.Direct2D1.Brush customDXBrush;
		
		int lastSelectedBar = 0;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "YieldCurveFree";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				CurveColor = Brushes.AliceBlue;
				TextColor = Brushes.DarkGray;
				GridColor = Brushes.SlateGray;
				
				MarginLeft = 50;
				MarginRight = 50;
			}
			else if (State == State.Configure)
			{
				foreach(string name in maturities)
				{
					AddDataSeries(name + " " + Expiry);
				}
			}
			else if (State == State.DataLoaded)
			{
				textFormat = new SharpDX.DirectWrite.TextFormat(Core.Globals.DirectWriteFactory, "Arial", 11);
				
				ChartControl.Properties.AreHGridLinesVisible = false;
				ChartControl.Properties.AreVGridLinesVisible = false;
			}
		}

		protected override void OnBarUpdate()
		{
		}
		
		protected override void OnRender(ChartControl chartControl, ChartScale chartScale)
		{
			base.OnRender(chartControl, chartScale);				
			if(Bars == null || Bars.Instrument == null || IsInHitTest ) { return; }
			if(CurrentBars[1] < 1 || CurrentBars[2] < 1 || CurrentBars[3] < 1 || CurrentBars[4] < 1) { return; }
			
			DrawCurveLines(chartControl, chartScale, 0, CurveColor);
			
			DrawVerticalLines(chartControl, chartScale);
		}
		
		
		public void DrawCurveLines(ChartControl chartControl, ChartScale chartScale, int mybar, Brush color)
		{
			try
			{			
				int previousPosition = GetPosition(chartControl, 0);
				//double previousValue = yields[0];
				int temp = CurrentBars[1] - mybar;
				double previousValue = BarsArray[1].GetClose(temp);
				float previousY = chartScale.GetYByValue(previousValue);
				
				using(SharpDX.Direct2D1.Brush myBrush =  color.ToDxBrush(RenderTarget))
				{
					
					for (int i = 1; i < 4; i++)
					{
						int currentPosition = GetPosition(chartControl, i);
						//double currentValue = yields[i];
						temp = CurrentBars[i+1] - mybar;
						double currentValue = BarsArray[i+1].GetClose(temp);				
						
						//Get y position based on value
						float y = chartScale.GetYByValue(currentValue);
						
						SharpDX.Vector2 start = new SharpDX.Vector2((float) previousPosition, (float) previousY);
						SharpDX.Vector2 end = new SharpDX.Vector2((float) currentPosition, (float) y);
						
						//draw the line
						RenderTarget.DrawLine(start, end, myBrush);
						
						previousPosition = currentPosition;
						previousValue = currentValue;
						previousY = y;
					}
				}
			}
			catch(Exception e)
			{
				Print("Error occured drawing curve lines");
			}
		}
		
		public void DrawVerticalLines(ChartControl chartControl, ChartScale chartScale)
		{
			
			using(SharpDX.Direct2D1.Brush myTextBrush = TextColor.ToDxBrush(RenderTarget))
			using(SharpDX.Direct2D1.Brush myGridBrush = GridColor.ToDxBrush(RenderTarget))
			{
				for (int i = 0; i < 4; i++)
				{
					int currentPosition = GetPosition(chartControl, i);
					
					SharpDX.Vector2 start = new SharpDX.Vector2((float) currentPosition, (float) 0);
					SharpDX.Vector2 end = new SharpDX.Vector2((float) currentPosition, (float) chartControl.ActualHeight);
					SharpDX.RectangleF textRectangle = new SharpDX.RectangleF(currentPosition, chartScale.GetYByValue(0), 50, 10);
									
					RenderTarget.DrawLine(start, end, myGridBrush);	
					
					//string yield = yields[i].ToString();
					string yield = BarsArray[i+1].GetClose(CurrentBars[i+1]).ToString();
					RenderTarget.DrawText(yield, textFormat, textRectangle, myTextBrush);
					//RenderTarget.FillRectangle(textRectangle, myBrush);
					
				}
			}
		}
		
		
		public override void OnCalculateMinMax()
		{
			if (CurrentBars[4] < 1)
				return;

			MinValue = -.25;
			MaxValue = BarsArray[4].GetClose(CurrentBars[4]) + .25;
		}
		
		
		public int GetPosition(ChartControl chartControl, int position)
		{
			//Left side
			if(position == 0)
			{
				return MarginLeft;
			}
			
			//Right side
			if(position == 3)
			{
				return chartControl.CanvasRight - MarginRight;
			}
			
			//Middle
			return MarginLeft + position * GetWidth(chartControl);
		}
		
		public int GetWidth(ChartControl chartControl)
		{
			return (chartControl.CanvasRight - MarginRight - MarginLeft) / 3;			
		}
		
		public int MarginLeft
		{ get; set; }
		
		public int MarginRight
		{ get; set; }
		
		[NinjaScriptProperty]
		public string Expiry
		{ get; set; }
		
		[XmlIgnore]
		[Display(Name="Color", Description="Yield Curve Line Color", Order=1, GroupName="Parameters")]
		public Brush CurveColor
		{ get; set; }

		[Browsable(false)]
		public string CurveColorSerializable
		{
			get { return Serialize.BrushToString(CurveColor); }
			set { CurveColor = Serialize.StringToBrush(value); }
		}		
		
		[XmlIgnore]
		[Display(Name="Text Color", Description="Text Color", Order=2, GroupName="Parameters")]
		public Brush TextColor
		{ get; set; }

		[Browsable(false)]
		public string TextColorSerializable
		{
			get { return Serialize.BrushToString(TextColor); }
			set { TextColor = Serialize.StringToBrush(value); }
		}			
		
		
		[XmlIgnore]
		[Display(Name="Gridline Color", Description="Grid Line Color", Order=3, GroupName="Parameters")]
		public Brush GridColor
		{ get; set; }

		[Browsable(false)]
		public string GridColorSerializable
		{
			get { return Serialize.BrushToString(GridColor); }
			set { GridColor = Serialize.StringToBrush(value); }
		}			
		
		[XmlIgnore]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Release and date", Description = "Release and date", GroupName = "Version", Order = 0)]
		public string VersionString
		{	
            get { return versionString; }
            set { ; }
		}	
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private SpeculatorSethFree.spsfYieldCurveFree[] cachespsfYieldCurveFree;
		public SpeculatorSethFree.spsfYieldCurveFree spsfYieldCurveFree(string expiry)
		{
			return spsfYieldCurveFree(Input, expiry);
		}

		public SpeculatorSethFree.spsfYieldCurveFree spsfYieldCurveFree(ISeries<double> input, string expiry)
		{
			if (cachespsfYieldCurveFree != null)
				for (int idx = 0; idx < cachespsfYieldCurveFree.Length; idx++)
					if (cachespsfYieldCurveFree[idx] != null && cachespsfYieldCurveFree[idx].Expiry == expiry && cachespsfYieldCurveFree[idx].EqualsInput(input))
						return cachespsfYieldCurveFree[idx];
			return CacheIndicator<SpeculatorSethFree.spsfYieldCurveFree>(new SpeculatorSethFree.spsfYieldCurveFree(){ Expiry = expiry }, input, ref cachespsfYieldCurveFree);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.SpeculatorSethFree.spsfYieldCurveFree spsfYieldCurveFree(string expiry)
		{
			return indicator.spsfYieldCurveFree(Input, expiry);
		}

		public Indicators.SpeculatorSethFree.spsfYieldCurveFree spsfYieldCurveFree(ISeries<double> input , string expiry)
		{
			return indicator.spsfYieldCurveFree(input, expiry);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.SpeculatorSethFree.spsfYieldCurveFree spsfYieldCurveFree(string expiry)
		{
			return indicator.spsfYieldCurveFree(Input, expiry);
		}

		public Indicators.SpeculatorSethFree.spsfYieldCurveFree spsfYieldCurveFree(ISeries<double> input , string expiry)
		{
			return indicator.spsfYieldCurveFree(input, expiry);
		}
	}
}

#endregion
